module DHT11(
	sys_clk,
	rst_n,
	dht11_req,
	dht11_done,
	dht11_error,
	tempH,
	tempL,
	humidityH,
	humidityL,
	dht11);

	//---Ports declearation: generated by Robei---
	input sys_clk;
	input rst_n;
	input dht11_req;
	output dht11_done;
	output dht11_error;
	output [7:0] tempH;
	output [7:0] tempL;
	output [7:0] humidityH;
	output [7:0] humidityL;
	inout dht11;

	wire sys_clk;
	wire rst_n;
	wire dht11_req;
	wire dht11_done;
	wire dht11_error;
	wire [7:0] tempH;
	wire [7:0] tempL;
	wire [7:0] humidityH;
	wire [7:0] humidityL;
	wire dht11;

	//----Code starts here: integrated by Robei-----
	//ʱΪ50MHZ20ns
	localparam	TIME18ms = 'd1000_099;    //ʼ̬18ms,900_000ʱڣʵӳʱ䡣
	localparam	TIME35us = 'd1_750;  //ݴУ0ߵĳ
	
	
	
	localparam S_IDLE			 =		'd0;  //̬
	localparam S_START_FPGA	 =		'd1;  //FPGAɼݿʼ
	localparam S_START_DHT11 =    'd2;  //DHT11ʼӦ
	localparam S_DATA			 =		'd3;	//ݴ
	localparam S_STOP			 =		'd4;  //ݽ
	localparam S_DOEN			 =		'd5;  //ݲɼ
	
	
	reg[2:0]		state , next_state;
	
	reg[22:0]	DHT11_Cnt;		//ʱ
	reg[5:0]		DHT11Bit_Cnt;  // dht11bit
	
	reg[39:0]	dht11_data;
	
	
	reg 			dht11_d0 , dht11_d1;
	wire 			dht11_negedge;  
	
	
	//dht11±   
	assign dht11_negedge = (~dht11_d0) & dht11_d1;
	
	assign dht11 = (state == S_START_FPGA && (DHT11_Cnt <= TIME18ms)) ? 1'b0 : 1'bz;
	assign dht11_done = (state == S_DOEN) ? 1'b1 : 1'b0;
	
	
	assign tempH 		= dht11_data[23:16];
	assign tempL 		= dht11_data[15:8];
	
	assign humidityH	= dht11_data[39:32];
	assign humidityL 	= dht11_data[31:24];
	
	
	
	always@(posedge sys_clk or negedge rst_n)
	begin
		if(rst_n == 1'b0)
		begin
			dht11_d0 <= 1'b1;
			dht11_d1 <= 1'b1;
		end
		else 
		begin
			dht11_d0 <= dht11;
			dht11_d1 <= dht11_d0;
		end
	end
	
	
	
	always@(posedge sys_clk or negedge rst_n)
	begin
		if(rst_n == 1'b0)
			state <= S_IDLE;
		else
			state <= next_state;
	end
	
	
	always@(*)
	begin
		case(state)
		S_IDLE:
			if(dht11_req == 1'b1)   //ݲɼ뿪ʼ̬
				next_state <= S_START_FPGA;
			else
				next_state <= S_IDLE;
		S_START_FPGA:                        
			if((DHT11_Cnt >= TIME18ms) && dht11_negedge == 1'b1)   //FPGA
				next_state <= S_START_DHT11;
			else
				next_state <= S_START_FPGA;
		S_START_DHT11:
			if((DHT11_Cnt > TIME35us) && dht11_negedge == 1'b1)  //ʱһʱͨжdht11ߵ½أǷӦ
				next_state <= S_DATA;
			else
				next_state <= S_START_DHT11;
		S_DATA:
			if(DHT11Bit_Cnt == 'd39 && dht11_negedge == 1'b1)  //յ40bitݺ󣬽ֹ̬ͣ
				next_state <= S_STOP;
			else
				next_state <= S_DATA;
		S_STOP:
			if(DHT11_Cnt == TIME35us + TIME35us)  //ݴɺ󣬵ȴ50us,70us
				next_state <= S_DOEN;
			else
				next_state <= S_STOP;
		S_DOEN:
			next_state <= S_IDLE;
				
		default: next_state <= S_IDLE;
		endcase
	end
	
	
	/*ģ*/
	always@(posedge sys_clk or negedge rst_n)
	begin
		if(rst_n == 1'b0)
			DHT11_Cnt <= 'd0;
		else if(state != next_state)      //״̬任ʱ򣬼ʱ0
			DHT11_Cnt <= 'd0; 
		else if(state == S_START_FPGA)   
			DHT11_Cnt <= DHT11_Cnt + 1'b1;
		else if(state == S_START_DHT11)
			DHT11_Cnt <= DHT11_Cnt + 1'b1;
		else if(state == S_DATA && dht11_negedge == 1'b0)   //ݵʱֻҪڸߵƽʱ
			DHT11_Cnt <= DHT11_Cnt + 1'b1;
		else if(state == S_STOP)
			DHT11_Cnt <= DHT11_Cnt + 1'b1;
		else
			DHT11_Cnt <= 'd0;		
	end
	
	
	
	/*ݴ洢*/
	always@(posedge sys_clk or negedge rst_n)
	begin
		if(rst_n == 1'b0)
			dht11_data <= 'd0;
		else if(state == S_DATA)
			if((DHT11_Cnt <= (TIME35us + 'd2500)) && dht11_negedge == 1'b1) //'d3000Ϊ͵ƽʱ䣬ߵƽʱ35usΪ0
				dht11_data <= {dht11_data[38:0],1'b0};
			else if(dht11_negedge == 1'b1)
				dht11_data <= {dht11_data[38:0],1'b1};
			else
				dht11_data <= dht11_data;
		else
			dht11_data <= dht11_data;
	end
	
	/*bit*/
	always@(posedge sys_clk or negedge rst_n)
	begin
		if(rst_n == 1'b0)
			DHT11Bit_Cnt <= 'd0;
		else if(state == S_DATA && dht11_negedge == 1'b1)
			DHT11Bit_Cnt <= DHT11Bit_Cnt + 1'b1;
		else if(state == S_DOEN)   //bit
			DHT11Bit_Cnt <= 'd0;
		else
			DHT11Bit_Cnt <= DHT11Bit_Cnt;
	end
	
	
	
endmodule    //DHT11

